# -*- coding:utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Mattia Verga <mattia.verga@protonmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

"""Setup review_stats.py logging."""

from logging.handlers import SMTPHandler

import logging
from . import log


def setup_logging(console_level, config_file, systemd_log):
    """Setup logging with one or more handlers."""
    log.setLevel('DEBUG')

    # Setup a console handler
    console_handler = logging.StreamHandler()
    console_format = logging.Formatter('%(asctime)s %(name)s %(levelname)-8s %(message)s')
    console_handler.setFormatter(console_format)
    console_handler.setLevel(console_level)

    log.addHandler(console_handler)

    if systemd_log:
        # Setup a rotating logfile
        from systemd import journal

        logfile_handler = journal.JournalHandler()
        logfile_format = logging.Formatter('%(asctime)s %(name)s %(levelname)-8s %(message)s')
        logfile_handler.setFormatter(logfile_format)
        logfile_handler.setLevel(config_file.get('review-stats-logging', 'journal_level',
                                                 fallback='INFO'))
        log.addHandler(logfile_handler)

    if 'mail_server' in config_file['review-stats-logging']:
        try:
            # Setup a smtp logger
            mail_server = config_file.get('review-stats-logging', 'mail_server').split(':')
            to_list = [mail for mail in config_file.get('review-stats-logging',
                                                        'mail_to',
                                                        fallback='').split(',')]
            if ('mail_secure' in config_file['review-stats-logging'] and
                    config_file['review-stats-logging']['mail_secure'] == 'true'):
                secure = ()
                credentials = (config_file.get('review-stats-logging', 'mail_username'),
                               config_file.get('review-stats-logging', 'mail_password'))
            else:
                secure = None
                credentials = None
            mail_handler = SMTPHandler(mailhost=mail_server,
                                       fromaddr=config_file.get('review-stats-logging',
                                                                'mail_from'),
                                       toaddrs=to_list,
                                       subject='review-stats script error',
                                       credentials=credentials,
                                       secure=secure)
            mail_format = logging.Formatter('%(asctime)s %(name)s %(levelname)-8s %(message)s')
            mail_handler.setFormatter(mail_format)
            mail_handler.setLevel(config_file.get('review-stats-logging', 'mail_level',
                                                  fallback='ERROR'))
            log.addHandler(mail_handler)
        except Exception:
            log.warning('Unable to use SMTP logging', exc_info=1)
