# -*- coding:utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Mattia Verga <mattia.verga@protonmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

"""Models for review_stats.py script."""

from datetime import datetime

from . import DEADREVIEW, LEGAL, NEEDSPONSOR


class ReviewTicket():
    """Wraps the Bug class with useful things for review_stats.

    Since python-bugzilla doesn't expose the bugzilla.bug.Bug class, it's not
    possible to use class inheritance.
    """
    def __init__(self, bug, blockers):
        self.bug = bug

        self.blocked = []
        self.dependencies_rejected = []
        for tk_blocker in bug.depends_on:
            for blocker in blockers:
                if tk_blocker == blocker.id and not blocker.status == 'CLOSED':
                    self.blocked.append(blocker)
                elif (tk_blocker == blocker.id and blocker.status == 'CLOSED'
                      and blocker.resolution == 'WONTFIX'):
                    self.dependencies_rejected.append(blocker)

    @property
    def assignee(self):
        """Shorten the long 'nobody's working on it' string."""
        if self.bug.assigned_to in ["Nobody's working on this, feel free to take it",
                                    "nobody@fedoraproject.org"]:
            return '(Nobody)'
        return self.bug.assigned_to

    @property
    def approved(self):
        """A flag to mark approved tickets on their way to repositories."""
        if self.bug.status in ['POST', 'MODIFIED', 'ON_QA'] and self.review_flag_status == '+':
            return True
        else:
            return False

    @property
    def created(self):
        """Get a useful datetime.datetime object for creation time."""
        return datetime.strptime(self.bug.creation_time.value, "%Y%m%dT%H:%M:%S")

    @property
    def dead_review(self):
        """A flag for tickets blocking FE-DEADREVIEW."""
        if DEADREVIEW in self.bug.blocks:
            return True
        else:
            return False

    @property
    def dependencies_on_qa(self):
        """Look if all ticket dependencies are on QA."""
        for blocker in self.blocked:
            if blocker.status != 'ON_QA':
                return False
        return True

    @property
    def fail_build(self):
        """A flag for tickets which fail build."""
        if 'buildfails' in self.bug.whiteboard.lower():
            return True
        else:
            return False

    @property
    def hidden(self):
        """A quick bool for mark the ticket as hidden."""
        return (not self.dependencies_on_qa or self.dead_review or self.fail_build or
                self.legal_issues or self.needinfo or self.not_ready or self.stalled)

    @property
    def last_modified(self):
        """Get a useful datetime.datetime object for last_change_time."""
        return datetime.strptime(self.bug.last_change_time.value, "%Y%m%dT%H:%M:%S")

    @property
    def legal_issues(self):
        """A flag for tickets with legal issues."""
        if LEGAL in self.bug.blocks:
            return True
        else:
            return False

    @property
    def needinfo(self):
        """A flag for tickets waiting for packager response."""
        for flag in self.bug.flags:
            if (flag['name'] == 'needinfo'
                    and flag['status'] == '?'
                    and flag['requestee'] == self.bug.creator):
                return True
        return False

    @property
    def needsponsor(self):
        """A flag for tickets in need of a sponsor."""
        if NEEDSPONSOR in self.bug.blocks:
            return True
        else:
            return False

    @property
    def not_ready(self):
        """A flag for tickets not yet ready for being reviewed."""
        if 'notready' in self.bug.whiteboard.lower():
            return True
        else:
            return False

    @property
    def really_assigned(self):
        """Check if the Ticket is **really** ASSIGNED.

        Do not rely only on bug status, check if the assignee is set.
        """
        if self.bug.status == 'ASSIGNED' and self.bug.assigned_to != 'nobody@fedoraproject.org':
            return True
        else:
            return False

    @property
    def review_flag_status(self):
        """Return the value of `fedora-review` flag."""
        for flag in self.bug.flags:
            if flag['name'] == 'fedora-review':
                return flag['status']
        return ''

    @property
    def stalled(self):
        """A flag for tickets which are stalled.

        Theoretically, these should also have needinfo set...
        """
        wb = self.bug.whiteboard.lower()
        if 'stalledsubmitter' in wb or 'awaitingsubmitter' in wb:
            return True
        else:
            return False

    @property
    def trivial(self):
        """A flag for tickets marked trivial."""
        if 'trivial' in self.bug.whiteboard.lower():
            return True
        else:
            return False
